package io.endeios.stocksui.utils;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;

public class TestUtils {
    public static String getAccessTokenUsingPasswordGrant(String tokenEndpointUrl, PasswordGrant passwordGrant) throws IOException {
        JsonNode answer = getTokenUsingPasswordGrant(
                tokenEndpointUrl,
                passwordGrant
        );
        return extractAccessToken(answer);
    }

    public static String extractAccessToken(JsonNode answer) {
        return answer.get("access_token").asText();
    }

    public static PasswordGrant getTestPasswordGrant() {
        String clientId = "stocks-ui";
        String clientSecret = "";
        String desiredScope = "openid";
        String username = "trader";
        String password = "";
        PasswordGrant passwordGrant = new PasswordGrant(clientId, clientSecret, desiredScope, username, password);
        return passwordGrant;
    }

    public static JsonNode getTokenUsingPasswordGrant(String tokenEndpointUrl, PasswordGrant passwordGrant) throws IOException {
        HttpURLConnection conn = createConnection(tokenEndpointUrl);
        sendPostRequest(conn, passwordGrant.clientId(), passwordGrant.clientSecret(), passwordGrant.desiredScope(), passwordGrant.username(), passwordGrant.password());
        return processResponse(conn);
    }

    private static HttpURLConnection createConnection(String tokenEndpointUrl) throws IOException {
        URL url = new URL(tokenEndpointUrl);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("POST");
        conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
        return conn;
    }

    private static void sendPostRequest(HttpURLConnection conn, String clientId, String clientSecret, String desiredScope, String username, String password) throws IOException {
        String postData = "grant_type=password&" +
                "client_id=" + clientId + "&" +
                "client_secret=" + clientSecret + "&" +
                "scope=" + desiredScope + "&" +
                "username=" + username + "&" +
                "password=" + password;

        conn.setDoOutput(true);
        OutputStream os = conn.getOutputStream();
        os.write(postData.getBytes());
        os.flush();
        os.close();
    }

    private static JsonNode processResponse(HttpURLConnection conn) throws IOException {
        BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
        String output;
        StringBuilder response = new StringBuilder();

        while ((output = in.readLine()) != null) {
            response.append(output);
        }
        in.close();

        ObjectMapper objectMapper = new ObjectMapper();
        return objectMapper.readTree(response.toString());
    }
}
